
module IdeaLattice
  class Lattice
    include Enumerable
    def initialize(top)
      @top = top
      @pages = { }
      @rest = [top]
    end
    def next?
      while @pages.keys.include?(@rest[0])
        @rest.shift
      end
      @rest.size > 0
    end
    def next_page
      @rest[0]
    end
    def add(pages)
      @rest.concat pages
      @pages[@rest[0]] = pages
    end
    def size
      @pages.size
    end
    def [](s)
      @pages[s]
    end
    def each
      @pages.each do |page, links|
        yield page, links
      end
    end
  end
end

if $0 == __FILE__
  if ARGV.length != 1
    $stderr.puts("usage: ruby lattice.rb marshaled")
    exit(1)
  end
  lattice = nil
  File.open(ARGV[0], 'r') do |f|
    lattice = Marshal.load(f)
  end
  puts "size=#{lattice.size}"
end
