# -*- coding: utf-8 -*-
require 'idealattice/lattice'
require 'rubigraph'
require 'iconv'
require 'cgi'

class SolarSystem
  include Rubigraph

  def initialize(lattice, max, enc = nil)
    @lattice = lattice
    @max = max
    @enc = enc
    Rubigraph.init        # initialize XML-RPC client.
    Rubigraph.clear
    @colors = [nil, nil, nil, nil]
  end

  def draw()
    vs = {}
    es = {}
    draw_internal(vs, es)
    r = vs.keys.sort do |a, b|
      @lattice[a].size <=> @lattice[b].size
    end
    0.upto(4) do |i|
      vs[r[i]].fontcolor = '#ffff00'
      vs[r[i]].color = '#ffff00'
    end
  end

  def draw_internal(vs, es)
    @lattice.each do |name, lks|
      return if vs.size > @max
      next if vs[name]
      v0 = new_vertex(name, vs)
      lks.each do |dst|
        next if dst == name
        v = vs[dst]
        unless v
          v = new_vertex(dst, vs)
        end
        next if es[name + dst]
        Edge.new(v, v0)
      end
    end
  end

  def new_vertex(name, hash)
    v = Vertex.new
    hash[name] = v
    v.label = conv(name)
    v
  end

  def conv(s)
    n = CGI.unescape(s)
    if @enc
      n = Iconv.iconv('utf-8', @enc, n)
      puts n
    end
    n
  end
end

